"use strict";
/**
 * Utilities to derive comments from various sources
 * @module
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommentSourceType = exports.deriveComment = exports.stats = void 0;
const lodash_1 = __importDefault(require("lodash"));
const typedoc_1 = require("typedoc");
exports.stats = lodash_1.default.fill(new Array(5), 0);
/**
 * Tries to figure out a comment for a command
 * @param knownMethods Known method map
 * @param command Name of method
 * @param refl Method reflection, if any
 * @param extraComment Comment from somewhere else (typically the method map), if any
 * @returns A `Comment`, if one can be found
 * @internal
 */
function deriveComment(command, knownMethods, refl, extraComment) {
    const commentFinders = [
        {
            /**
             * @returns The comment on the method itself
             */
            getter: () => refl?.comment?.summary,
            commentSource: CommentSourceType.Method,
        },
        {
            /**
             * @returns The comment from the inherited method
             */
            getter: () => refl?.inheritedFrom?.reflection?.comment?.summary,
            commentSource: CommentSourceType.MethodInheritedFrom,
        },
        {
            /**
             * @returns The comment from the method's signature
             */
            getter: () => refl?.getAllSignatures().find((sig) => sig.comment?.summary)?.comment?.summary,
            commentSource: CommentSourceType.MethodSignature,
        },
        {
            /**
             *
             * @returns The comment from the interface's method signature
             */
            getter: () => knownMethods
                ?.get(command)
                ?.method.getAllSignatures()
                .find((sig) => sig.comment?.summary)?.comment?.summary,
            commentSource: CommentSourceType.BuiltinMethodSignature,
        },
        {
            /**
             *
             * @returns A comment probably from the method map itself
             */
            getter: () => extraComment?.summary,
            commentSource: CommentSourceType.OtherComment,
        },
    ];
    for (const [idx, { getter, commentSource }] of commentFinders.entries()) {
        const commentParts = getter();
        exports.stats[idx] = exports.stats[idx] + Number(Boolean(commentParts?.length));
        if (commentParts?.length) {
            return {
                comment: new typedoc_1.Comment(commentParts),
                commentSource,
            };
        }
    }
}
exports.deriveComment = deriveComment;
var CommentSourceType;
(function (CommentSourceType) {
    CommentSourceType["Method"] = "method";
    CommentSourceType["MethodInheritedFrom"] = "method-inherited-from";
    CommentSourceType["MethodSignature"] = "method-signature";
    CommentSourceType["BuiltinMethodSignature"] = "builtin-method-signature";
    CommentSourceType["OtherComment"] = "other-comment";
})(CommentSourceType = exports.CommentSourceType || (exports.CommentSourceType = {}));
//# sourceMappingURL=comment.js.map